@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
  *
- $Id: power-management.c,v 1.5 2005/02/24 08:54:02 kmaraas Exp $
+ $Id: power-management.c,v 1.6 2005/03/20 10:20:55 kmaraas Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -28,6 +28,7 @@
 #include <sys/file.h>
 #include <sys/ioctl.h>
 #include <sys/types.h>
+#include <sys/stat.h>
 #include <sys/sysctl.h>
 
 #include <err.h>
@@ -38,7 +39,6 @@
 #include <time.h>
 #include <unistd.h>
 
-#include <gnome.h> /* needed to make it through battstat.h */
 #include <panel-applet.h>
 #include <panel-applet-gconf.h>
 
