@@ -26,24 +26,6 @@
 
 #define UPDATE_TIMEOUT 100
 
-/* FIXME crashes, need to fix */
-static void
-applet_set_back_pixmap (PanelApplet *applet, GtkWidget *widget, GdkPixmap *pixmap)
-{
-	GtkStyle *new_style;
-
-        new_style = gtk_style_copy (widget->style);
-        gtk_style_ref (new_style);
-/* FIXME should this be done ? */
-/*
-        if (new_style->bg_pixmap [GTK_STATE_NORMAL])
-		g_object_unref (new_style->bg_pixmap [GTK_STATE_NORMAL]);
-*/
-        new_style->bg_pixmap [GTK_STATE_NORMAL] = pixmap;
-        gtk_widget_set_style (widget, new_style);
-        gtk_style_unref (new_style);
-}
-
 static void
 applet_back_change (PanelApplet			*a,
 		    PanelAppletBackgroundType	type,
@@ -51,74 +33,82 @@ applet_back_change (PanelApplet			*a,
 		    GdkPixmap			*pixmap,
 		    EyesApplet			*eyes_applet) 
 {
-	GtkRcStyle *rc_style = gtk_rc_style_new ();
+        /* taken from the TrashApplet */
+        GtkRcStyle *rc_style;
+        GtkStyle *style;
 
-	switch (type) {
-	case PANEL_PIXMAP_BACKGROUND:
-		gtk_widget_modify_style (GTK_WIDGET (eyes_applet->applet), rc_style);
-/* FIXME can't get to work */
-//		applet_set_back_pixmap (a, GTK_WIDGET (eyes_applet->applet), pixmap);
-		break;
+        /* reset style */
+        gtk_widget_set_style (GTK_WIDGET (eyes_applet->applet), NULL);
+        rc_style = gtk_rc_style_new ();
+        gtk_widget_modify_style (GTK_WIDGET (eyes_applet->applet), rc_style);
+        gtk_rc_style_unref (rc_style);
 
-	case PANEL_COLOR_BACKGROUND:
-		gtk_widget_modify_bg (GTK_WIDGET (eyes_applet->applet), GTK_STATE_NORMAL, color);
-		break;
+        switch (type) {
+                case PANEL_COLOR_BACKGROUND:
+                        gtk_widget_modify_bg (GTK_WIDGET (eyes_applet->applet),
+                                        GTK_STATE_NORMAL, color);
+                        break;
 
-	case PANEL_NO_BACKGROUND:
-		gtk_widget_modify_style (GTK_WIDGET (eyes_applet->applet), rc_style);
-		break;
+                case PANEL_PIXMAP_BACKGROUND:
+                        style = gtk_style_copy (GTK_WIDGET (
+						eyes_applet->applet)->style);
+                        if (style->bg_pixmap[GTK_STATE_NORMAL])
+                                g_object_unref
+                                        (style->bg_pixmap[GTK_STATE_NORMAL]);
+                        style->bg_pixmap[GTK_STATE_NORMAL] = g_object_ref
+                                (pixmap);
+                        gtk_widget_set_style (GTK_WIDGET (eyes_applet->applet),
+                                        style);
+                        g_object_unref (style);
+                        break;
 
-	default:
-		gtk_widget_modify_style (GTK_WIDGET (eyes_applet->applet), rc_style);
-		break;
-	}
+                case PANEL_NO_BACKGROUND:
+                default:
+                        break;
+        }
 
-	gtk_rc_style_unref (rc_style);
 }
 
 /* TODO - Optimize this a bit */
 static void 
 calculate_pupil_xy (EyesApplet *eyes_applet,
 		    gint x, gint y,
-		    gint *pupil_x, gint *pupil_y)
+		    gint *pupil_x, gint *pupil_y, GtkWidget* widget)
 {
         double angle;
         double sina;
         double cosa;
-        double nx;
-        double ny;
         double h;
         double temp;
-     
-        nx = x - ((double) eyes_applet->eye_width / 2);
-        ny = y - ((double) eyes_applet->eye_height / 2);
-       
-        h = hypot (nx, ny);
-        if (abs (h) 
+ 	 double nx, ny;
+
+	 gfloat xalign, yalign;
+	 gint width, height;
+
+	 width = GTK_WIDGET(widget)->allocation.width;
+	 height = GTK_WIDGET(widget)->allocation.height;
+	 gtk_misc_get_alignment(GTK_MISC(widget),  &xalign, &yalign);
+
+	 nx = x - MAX(width - eyes_applet->eye_width, 0) * xalign - eyes_applet->eye_width / 2;
+	 ny = y - MAX(height- eyes_applet->eye_height, 0) * yalign - eyes_applet->eye_height / 2;
+  
+	 h = hypot (nx, ny);
+        if (h < 0.5 || abs (h) 
             < (abs (hypot (eyes_applet->eye_height / 2, eyes_applet->eye_width / 2)) - eyes_applet->wall_thickness - eyes_applet->pupil_height)) {
-                *pupil_x = x;
-                *pupil_y = y;
+                *pupil_x = nx + eyes_applet->eye_width / 2;
+                *pupil_y = ny + eyes_applet->eye_height / 2;
                 return;
         }
         
-        angle = atan2 (nx, ny);
-        sina = sin (angle);
-        cosa = cos (angle);
-        
-        temp = hypot ((eyes_applet->eye_height / 2) * cosa, (eyes_applet->eye_width / 2)* sina);
-        *pupil_x = temp * sina;
-        *pupil_y = temp * cosa;
-       
-        temp = hypot ((eyes_applet->pupil_width / 2) * sina, (eyes_applet->pupil_height / 2)* cosa);
-        *pupil_x -= temp * sina;
-        *pupil_y -= temp * cosa;
-        
-        temp = hypot ((eyes_applet->wall_thickness / 2) * sina, (eyes_applet->wall_thickness / 2) * cosa);
-        *pupil_x -= temp * sina;
-        *pupil_y -= temp * cosa;
-        
-        *pupil_x += (eyes_applet->eye_width / 2);
-        *pupil_y += (eyes_applet->eye_height / 2);
+	 sina = nx / h; 
+	 cosa = ny / h;
+	
+        temp = hypot ((eyes_applet->eye_width / 2) * sina, (eyes_applet->eye_height / 2) * cosa);
+        temp -= hypot ((eyes_applet->pupil_width / 2) * sina, (eyes_applet->pupil_height / 2) * cosa);
+        temp -= hypot ((eyes_applet->wall_thickness / 2) * sina, (eyes_applet->wall_thickness / 2) * cosa);
+
+        *pupil_x = temp * sina + (eyes_applet->eye_width / 2);
+        *pupil_y = temp * cosa + (eyes_applet->eye_height / 2);
 }
 
 static void 
@@ -161,19 +151,21 @@ timer_cb (EyesApplet *eyes_applet)
         gint x, y;
         gint pupil_x, pupil_y;
         gint i;
-        
+
         for (i = 0; i < eyes_applet->num_eyes; i++) {
 		if (GTK_WIDGET_REALIZED (eyes_applet->eyes[i])) {
 			gtk_widget_get_pointer (eyes_applet->eyes[i], 
 						&x, &y);
-			if ((x != eyes_applet->pointer_last_x) || (y != eyes_applet->pointer_last_y)) { 
-				calculate_pupil_xy (eyes_applet, x, y, &pupil_x, &pupil_y);
+			if ((x != eyes_applet->pointer_last_x[i]) || (y != eyes_applet->pointer_last_y[i])) { 
+
+				calculate_pupil_xy (eyes_applet, x, y, &pupil_x, &pupil_y, eyes_applet->eyes[i]);
 				draw_eye (eyes_applet, i, pupil_x, pupil_y);
+	    	        
+			        eyes_applet->pointer_last_x[i] = x;
+			        eyes_applet->pointer_last_y[i] = y;
 			}
 		}
         }
-        eyes_applet->pointer_last_x = x;
-        eyes_applet->pointer_last_y = y;
         return TRUE;
 }
 
@@ -182,10 +174,6 @@ about_cb (BonoboUIComponent *uic,
 	  EyesApplet        *eyes_applet,
 	  const gchar       *verbname)
 {
-	GdkPixbuf	 *pixbuf;
-	GError		 *error = NULL;
-	gchar		 *file;
-        
         static const gchar *authors [] = {
 		"Dave Camp <campd@oit.edu>",
 		NULL
@@ -198,8 +186,6 @@ about_cb (BonoboUIComponent *uic,
 		NULL
 	};
 
-	const gchar *translator_credits = _("translator_credits");
-
 	if (eyes_applet->about_dialog) {
 		gtk_window_set_screen (GTK_WINDOW (eyes_applet->about_dialog),
 				       gtk_widget_get_screen (GTK_WIDGET (eyes_applet->applet)));
@@ -208,27 +194,19 @@ about_cb (BonoboUIComponent *uic,
 		return;
 	}
         
-	file = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_PIXMAP, "gnome-eyes.png", FALSE, NULL);
-	pixbuf = gdk_pixbuf_new_from_file (file, &error);
-	g_free (file);
-	
-	if (error) {
-		g_warning (G_STRLOC ": cannot open %s: %s", file, error->message);
-		g_error_free (error);
-	}
-	
-        eyes_applet->about_dialog = gnome_about_new (
-		_("Geyes"), VERSION,
-		_("Copyright (C) 1999 Dave Camp"),
-		_("A goofy little xeyes clone for the GNOME panel."),
-		authors,
-		documenters,
-		strcmp (translator_credits, "translator_credits") != 0 ? translator_credits : NULL,
-		pixbuf);
+        eyes_applet->about_dialog = gtk_about_dialog_new ();
+
+	g_object_set (eyes_applet->about_dialog,
+		      "name", _("Geyes"),
+		      "version", VERSION,
+		      "comments", _("A goofy little xeyes clone for the GNOME panel."),
+		      "copyright", _("Copyright (C) 1999 Dave Camp"),
+		      "authors", authors,
+		      "dokcumenters", documenters,
+		      "translation-credits", strcmp (_("translator_credits"), "translator_credits") != 0 ? _("translator_credits") : NULL,
+		      "logo-icon-name", "gnome-eyes",
+		      NULL);
 		
-	if (pixbuf)
-		gdk_pixbuf_unref (pixbuf);
-			
 	gtk_window_set_wmclass (GTK_WINDOW (eyes_applet->about_dialog), "geyes", "Geyes");
 	gtk_window_set_screen (GTK_WINDOW (eyes_applet->about_dialog),
 			       gtk_widget_get_screen (GTK_WIDGET (eyes_applet->applet)));
@@ -269,6 +247,8 @@ setup_eyes (EyesApplet *eyes_applet)
         gtk_box_pack_start (GTK_BOX (eyes_applet->vbox), eyes_applet->hbox, TRUE, TRUE, 0);
 
 	eyes_applet->eyes = g_new0 (GtkWidget *, eyes_applet->num_eyes);
+	eyes_applet->pointer_last_x = g_new0 (gint, eyes_applet->num_eyes);
+	eyes_applet->pointer_last_y = g_new0 (gint, eyes_applet->num_eyes);
 
         for (i = 0; i < eyes_applet->num_eyes; i++) {
                 eyes_applet->eyes[i] = gtk_image_new ();
@@ -298,6 +278,10 @@ setup_eyes (EyesApplet *eyes_applet)
 		}
 		
                 gtk_widget_realize (eyes_applet->eyes[i]);
+		
+		eyes_applet->pointer_last_x[i] = G_MAXINT;
+		eyes_applet->pointer_last_y[i] = G_MAXINT;
+		
 		draw_eye (eyes_applet, i,
 			  eyes_applet->eye_width / 2,
                           eyes_applet->eye_height / 2);
@@ -313,6 +297,8 @@ destroy_eyes (EyesApplet *eyes_applet)
 	eyes_applet->hbox = NULL;
 
 	g_free (eyes_applet->eyes);
+	g_free (eyes_applet->pointer_last_x);
+	g_free (eyes_applet->pointer_last_y);
 }
 
 static EyesApplet *
@@ -415,7 +401,7 @@ geyes_applet_fill (PanelApplet *applet)
 {
 	EyesApplet *eyes_applet;
 	
-	gnome_window_icon_set_default_from_file (GNOME_ICONDIR"/gnome-eyes.png");
+	gtk_window_set_default_icon_name ("gnome-eyes");
 	panel_applet_set_flags (applet, PANEL_APPLET_EXPAND_MINOR);
 	
         eyes_applet = create_eyes (applet);
@@ -428,7 +414,7 @@ geyes_applet_fill (PanelApplet *applet)
 		UPDATE_TIMEOUT, (GtkFunction) timer_cb, eyes_applet);
 			
 	panel_applet_setup_menu_from_file (eyes_applet->applet,
-                                           NULL,
+                                           DATADIR,
 				           "GNOME_GeyesApplet.xml",
                                            NULL,
 				           geyes_applet_menu_verbs,
