@@ -16,33 +16,41 @@
 
 #include "config.h"
 
-#include <gdk/gdkscreen.h>
-#include <gdk/gdkx.h>
-#include <gnome.h>
-#include <glade/glade.h>
-
-#include <libxklavier/xklavier_config.h>
-
 #include "gswitchit-applet.h"
-
 #include "libgswitchit/gswitchit_util.h"
 #include "libgswitchit/gswitchit_plugin_manager.h"
 
-#if 1
+#include <string.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <unistd.h>
-#endif
+
+#include <gdk/gdkscreen.h>
+#include <gdk/gdkx.h>
+#include <glade/glade.h>
+
+#include <libxklavier/xklavier_config.h>
 
 #define GROUPS_SUBMENU_PATH "/popups/popup/groups"
 
-static gboolean terminatedOnce = FALSE;
-static GSwitchItApplet *theAppletInstance = NULL;
+/* one instance for ALL applets */
+static GSwitchItAppletGlobals globals;
 
-static void GSwitchItAppletCmdProps (BonoboUIComponent * uic,
-				     GSwitchItApplet * sia,
-				     const gchar * verb);
+#define GSwitchItAppletFirstInstance() \
+	(!g_slist_length (globals.appletInstances))
+
+#define GSwitchItAppletLastInstance() \
+	(!g_slist_length (globals.appletInstances))
+
+#define ForAllApplets() \
+	{ \
+		GSList* cur; \
+		for (cur = globals.appletInstances; cur != NULL; cur = cur->next) { \
+			GSwitchItApplet * sia = (GSwitchItApplet*)cur->data;
+#define NextApplet \
+		} \
+	}
 
 static void GSwitchItAppletCmdCapplet (BonoboUIComponent * uic,
 				       GSwitchItApplet * sia,
@@ -68,10 +76,6 @@ static void GSwitchItAppletCleanupGroupsSubmenu (GSwitchItApplet * sia);
 
 static void GSwitchItAppletSetupGroupsSubmenu (GSwitchItApplet * sia);
 
-static GdkFilterReturn GSwitchItAppletWmProtocolsFilter (GdkXEvent * xev,
-							 GdkEvent * event,
-							 gpointer data);
-
 static void GSwitchItAppletFillNotebook (GSwitchItApplet * sia);
 
 static void GSwitchItAppletCleanupNotebook (GSwitchItApplet * sia);
@@ -86,7 +90,6 @@ static gboolean GSwitchItAppletKeyPressed (GtkWidget * widget,
 					   GSwitchItAppletConfig * config);
 
 static const BonoboUIVerb gswitchitAppletMenuVerbs[] = {
-	BONOBO_UI_UNSAFE_VERB ("Props", GSwitchItAppletCmdProps),
 	BONOBO_UI_UNSAFE_VERB ("Capplet", GSwitchItAppletCmdCapplet),
 	BONOBO_UI_UNSAFE_VERB ("Plugins", GSwitchItAppletCmdPlugins),
 	BONOBO_UI_UNSAFE_VERB ("About", GSwitchItAppletCmdAbout),
@@ -128,89 +131,110 @@ GSwitchItAppletReinitUi (GSwitchItApplet * sia)
 	/* also, update tooltips */
 	currentState = XklGetCurrentState ();
 	if (currentState->group >= 0) {
-		pname = sia->groupNames[currentState->group];
+		pname = g_slist_nth_data (globals.groupNames, currentState->group);
 		GSwitchItAppletSetTooltip (sia, pname);
 	}
 }
 
+/* Should be called once for all applets */
+static void
+GSwitchItConfigChanged (GConfClient * client,
+			guint cnxn_id,
+			GConfEntry * entry)
+{
+	XklDebug (100,
+		  "General configuration changed in GConf - reiniting...\n");
+	GSwitchItConfigLoadFromGConf (&globals.config);
+	GSwitchItConfigActivate (&globals.config);
+	ForAllApplets ()
+		GSwitchItAppletReinitUi (sia);
+	NextApplet
+}
+
+/* Should be called once for all applets */
 static void
 GSwitchItAppletConfigChanged (GConfClient * client,
 			      guint cnxn_id,
-			      GConfEntry * entry, GSwitchItApplet * sia)
+			      GConfEntry * entry)
 {
+	GSList* enabledPlugins = NULL;
+
 	XklDebug (100,
 		  "Applet configuration changed in GConf - reiniting...\n");
+	GSwitchItAppletConfigLoadFromGConf (&globals.appletConfig);
+	GSwitchItAppletConfigUpdateImages (&globals.appletConfig, &globals.kbdConfig);
+	GSwitchItAppletConfigActivate (&globals.appletConfig);
+	
+	GSwitchItPluginManagerTogglePlugins (&globals.pluginManager,
+					     &globals.pluginContainer,
+					     globals.appletConfig.enabledPlugins);
 
-	GSwitchItAppletConfigLoad (&sia->appletConfig);
-	GSwitchItAppletConfigUpdateImages (&sia->appletConfig,
-					   &sia->xkbConfig);
-	GSwitchItAppletConfigActivate (&sia->appletConfig);
-	GSwitchItPluginManagerTogglePlugins (&sia->pluginManager,
-					     &sia->pluginContainer,
-					     sia->appletConfig.
-					     enabledPlugins);
-	GSwitchItAppletReinitUi (sia);
-}
-
-static int
-GSwitchItAppletWindowCallback (Window win, Window parent,
-			       GSwitchItApplet * sia)
-{
-	return GSwitchItPluginManagerWindowCreated (&sia->pluginManager,
-						    win, parent);
+	ForAllApplets ()
+		GSwitchItAppletReinitUi (sia);
+	NextApplet
 }
 
+/* Should be called once for all applets */
 static void
-GSwitchItAppletXkbConfigCallback (GSwitchItApplet * sia)
+GSwitchItAppletKbdConfigCallback (void)
 {
 	XklDebug (100,
 		  "XKB configuration changed on X Server - reiniting...\n");
-	GSwitchItXkbConfigLoadCurrent (&sia->xkbConfig);
-	GSwitchItAppletConfigUpdateImages (&sia->appletConfig,
-					   &sia->xkbConfig);
-	GSwitchItAppletConfigLoadGroupDescriptionsUtf8 (&sia->appletConfig,
-							sia->groupNames);
 
-	GSwitchItAppletReinitUi (sia);
+	GSwitchItKbdConfigLoadFromXCurrent (&globals.kbdConfig);
+	GSwitchItAppletConfigUpdateImages (&globals.appletConfig, &globals.kbdConfig);
+
+	while (globals.groupNames != NULL) {
+		GSList * nn = globals.groupNames;
+		globals.groupNames = g_slist_remove_link (globals.groupNames, nn);
+		g_free (nn->data);
+		g_slist_free_1 (nn);
+	}
+	globals.groupNames = GSwitchItConfigLoadGroupDescriptionsUtf8 (&globals.config);
+
+	ForAllApplets ()
+		GSwitchItAppletReinitUi (sia);
+	NextApplet
 }
 
+/* Should be called once for all applets */
 static void
-GSwitchItAppletStateCallback (XklStateChange
-			      changeType,
-			      int group, Bool restore,
-			      GSwitchItApplet * sia)
+GSwitchItAppletStateCallback (XklStateChange changeType,
+			      int group, Bool restore)
 {
 	XklDebug (150, "group is now %d, restore: %d\n", group, restore);
 
 	if (changeType == GROUP_CHANGED) {
-		GSwitchItPluginManagerGroupChanged (&sia->pluginManager,
+		GSwitchItPluginManagerGroupChanged (&globals.pluginManager,
 						    group);
-		XklDebug (200, "do repaint\n");
-		GSwitchItAppletRevalidateGroup (sia, group);
+		ForAllApplets ()
+			XklDebug (200, "do repaint\n");
+			GSwitchItAppletRevalidateGroup (sia, group);
+		NextApplet
 	}
 }
 
+/* Should be called once for all applets */
 GdkFilterReturn
-GSwitchItAppletFilterXEvt (GdkXEvent * xev,
-			   GdkEvent * event, GSwitchItApplet * sia)
+GSwitchItAppletFilterXEvt (GdkXEvent * xev, GdkEvent * event)
 {
 	XEvent *xevent = (XEvent *) xev;
-	Display *display = xevent->xany.display;
+
 	XklFilterEvents (xevent);
 	switch (xevent->type) {
 	case ReparentNotify:
 		{
 			XReparentEvent *rne = (XReparentEvent *) xev;
-			GtkWidget *w1;
-			GdkWindow *w;
-			w1 = gtk_widget_get_ancestor (sia->applet,
-						      GTK_TYPE_WINDOW);
-			if (w1 == NULL)
-				break;
-			w = w1->window;
-			if (w == NULL || GDK_WINDOW_XID (w) != rne->window)
-				break;
-			XklSetTransparent (GDK_WINDOW_XID (w), TRUE);
+
+			ForAllApplets ()
+				GdkWindow * w = sia->appletAncestor->window;
+				/* compare the applet's parent window with the even window */
+				if (w != NULL && GDK_WINDOW_XID (w) == rne->window) {
+					/* if so - make it transparent...*/
+					XklSetTransparent (rne->window, TRUE);
+					break; /* once is enough */
+				}
+			NextApplet
 		}
 		break;
 	}
@@ -218,26 +242,6 @@ GSwitchItAppletFilterXEvt (GdkXEvent * xev,
 	return GDK_FILTER_CONTINUE;
 }
 
-GdkFilterReturn
-GSwitchItAppletWmProtocolsFilter (GdkXEvent *
-				  xev, GdkEvent * event, gpointer data)
-{
-	XEvent *xevent = (XEvent *) xev;
-	Display *display = xevent->xany.display;
-	if ((Atom) xevent->xclient.data.l[0] ==
-	    XInternAtom (display, "_NET_WM_PING", FALSE)) {
-		static long lastTimestamp = -1;
-		long thisTimestamp = xevent->xclient.data.l[1];
-		/* allow only events we have not processed yet */
-		if (lastTimestamp == thisTimestamp) {
-			return GDK_FILTER_REMOVE;
-		}
-		lastTimestamp = thisTimestamp;
-	}
-
-	return GDK_FILTER_CONTINUE;
-}
-
 void
 GSwitchItAppletRevalidate (GSwitchItApplet * sia)
 {
@@ -250,10 +254,9 @@ GSwitchItAppletRevalidate (GSwitchItApplet * sia)
 static void
 GSwitchItAppletCleanupNotebook (GSwitchItApplet * sia)
 {
-	GtkNotebook *notebook =
-	    GTK_NOTEBOOK (GTK_BIN (sia->applet)->child);
-
 	int i;
+	GtkNotebook *notebook = GTK_NOTEBOOK (sia->notebook);
+
 	/* Do not remove the first page! It is the default page */
 	for (i = gtk_notebook_get_n_pages (notebook); --i > 0;) {
 		gtk_notebook_remove_page (notebook, i);
@@ -263,16 +266,19 @@ GSwitchItAppletCleanupNotebook (GSwitchItApplet * sia)
 static GtkWidget *
 GSwitchItAppletPrepareDrawing (GSwitchItApplet * sia, int group)
 {
-	GdkPixbuf *image = sia->appletConfig.images[group];
+	gpointer pimage;
+	GdkPixbuf *image;
 	GdkPixbuf *scaled;
 	PanelAppletOrient orient;
 	int psize, xsize = 0, ysize = 0;
 	double xyratio;
+	pimage = g_slist_nth_data (globals.appletConfig.images, group);
 	sia->ebox = gtk_event_box_new ();
-	if (sia->appletConfig.showFlags) {
+	if (globals.appletConfig.showFlags) {
 		GtkWidget *flagImg;
-		if (image == NULL)
+		if (pimage == NULL)
 			return NULL;
+		image = GDK_PIXBUF (pimage);
 		orient =
 		    panel_applet_get_orient (PANEL_APPLET (sia->applet));
 		psize =
@@ -299,19 +305,18 @@ GSwitchItAppletPrepareDrawing (GSwitchItApplet * sia, int group)
 		flagImg = gtk_image_new_from_pixbuf (scaled);
 		gtk_container_add (GTK_CONTAINER (sia->ebox), flagImg);
 		g_object_unref (G_OBJECT (scaled));
-		//sia->ebox = NULL;	// not used in this case!
 	} else {
 		char *layoutName;
 		char *allocLayoutName = NULL;
 		XklConfigItem configItem;
 		GtkWidget *align, *label;
 
-		if (XklMultipleLayoutsSupported ()) {
+		if (XklGetBackendFeatures() & XKLF_MULTIPLE_LAYOUTS_SUPPORTED) {
 			char *fullLayoutName =
-			    (char *) g_slist_nth_data (sia->xkbConfig.
-						       layouts, group);
+			    (char *) g_slist_nth_data (globals.kbdConfig.layouts, 
+						       group);
 			char *variantName;
-			if (!GSwitchItConfigSplitItems
+			if (!GSwitchItKbdConfigSplitItems
 			    (fullLayoutName, &layoutName, &variantName))
 				/* just in case */
 				layoutName = fullLayoutName;
@@ -329,7 +334,7 @@ GSwitchItAppletPrepareDrawing (GSwitchItApplet * sia, int group)
 				}
 			}
 		} else
-			layoutName = sia->groupNames[group];
+			layoutName = g_slist_nth_data (globals.groupNames, group);
 		align = gtk_alignment_new (0.5, 0.5, 1.0, 1.0);
 		label = gtk_label_new (layoutName);
 		if (allocLayoutName != NULL)
@@ -355,10 +360,9 @@ GSwitchItAppletPrepareDrawing (GSwitchItApplet * sia, int group)
 static void
 GSwitchItAppletFillNotebook (GSwitchItApplet * sia)
 {
-	GtkNotebook *notebook;
 	int grp;
 	int totalGroups = XklGetNumGroups ();
-	notebook = GTK_NOTEBOOK (GTK_BIN (sia->applet)->child);
+	GtkNotebook *notebook = GTK_NOTEBOOK (sia->notebook);
 
 	for (grp = 0; grp < totalGroups; grp++) {
 		GtkWidget *page, *decoratedPage;
@@ -368,12 +372,12 @@ GSwitchItAppletFillNotebook (GSwitchItApplet * sia)
 		page = GSwitchItAppletPrepareDrawing (sia, grp);
 
 		decoratedPage =
-		    GSwitchItPluginManagerDecorateWidget (&sia->
-							  pluginManager,
+		    GSwitchItPluginManagerDecorateWidget (&globals.pluginManager,
 							  page, grp,
-							  sia->
-							  groupNames[grp],
-							  &sia->xkbConfig);
+							  g_slist_nth_data (
+								globals.groupNames,
+								grp),
+							  &globals.kbdConfig);
 
 		gtk_notebook_append_page (notebook,
 					  decoratedPage ==
@@ -388,13 +392,11 @@ void
 GSwitchItAppletRevalidateGroup (GSwitchItApplet * sia, int group)
 {
 	const char *pname;
-	GtkNotebook *notebook;
 	XklDebug (200, "Revalidating for group %d\n", group);
-	notebook = GTK_NOTEBOOK (GTK_BIN (sia->applet)->child);
 
-	gtk_notebook_set_current_page (notebook, group + 1);
+	gtk_notebook_set_current_page (GTK_NOTEBOOK (sia->notebook), group + 1);
 
-	pname = sia->groupNames[group];
+	pname = g_slist_nth_data (globals.groupNames, group);
 	GSwitchItAppletSetTooltip (sia, pname);
 }
 
@@ -408,48 +410,56 @@ GSwitchItAppletChangePixelSize (PanelApplet *
 	GSwitchItAppletRevalidate (sia);
 }
 
+static void 
+GSwitchItAppletSetBackground(PanelAppletBackgroundType type,
+			     GtkRcStyle * rc_style,
+			     GtkWidget * w, 
+			     GdkColor * color,
+			     GdkPixmap * pixmap)
+{
+	GtkStyle *style;
+
+	gtk_widget_set_style (GTK_WIDGET (w), NULL);
+	gtk_widget_modify_style (GTK_WIDGET (w), rc_style);
+
+	switch (type)
+	{
+	case PANEL_NO_BACKGROUND:
+	        break;
+	case PANEL_COLOR_BACKGROUND:
+	        gtk_widget_modify_bg (GTK_WIDGET (w),
+	                              GTK_STATE_NORMAL, color);
+		break;
+	case PANEL_PIXMAP_BACKGROUND:
+		style = gtk_style_copy (w->style);
+		if (style->bg_pixmap[GTK_STATE_NORMAL])
+			g_object_unref (style->bg_pixmap[GTK_STATE_NORMAL]);
+		style->bg_pixmap[GTK_STATE_NORMAL] = g_object_ref (pixmap);
+		gtk_widget_set_style (w, style);
+		g_object_unref (style);
+		break;
+	}
+	/* go down */
+	if (GTK_IS_CONTAINER (w))
+	{
+		GList * child = gtk_container_get_children (GTK_CONTAINER (w));
+		while (child != NULL)
+		{
+			GSwitchItAppletSetBackground (type, rc_style,
+			     GTK_WIDGET (child->data), color, pixmap);
+			child = child->next;
+		}
+	}
+}
+
 static void
-GSwitchItAppletChangeBackground (PanelApplet *
-				 widget, PanelAppletBackgroundType type,
+GSwitchItAppletChangeBackground (PanelApplet * widget, 
+				 PanelAppletBackgroundType type,
 				 GdkColor * color, GdkPixmap * pixmap,
 				 GSwitchItApplet * sia)
 {
 	GtkRcStyle *rc_style = gtk_rc_style_new ();
-
-	switch (type) {
-	case PANEL_PIXMAP_BACKGROUND:
-		if (sia->ebox != NULL)
-			gtk_widget_modify_style (GTK_WIDGET (sia->ebox),
-						 rc_style);
-		gtk_widget_modify_style (GTK_WIDGET (sia->applet),
-					 rc_style);
-		break;
-
-	case PANEL_COLOR_BACKGROUND:
-		if (sia->ebox != NULL)
-			gtk_widget_modify_bg (GTK_WIDGET (sia->ebox),
-					      GTK_STATE_NORMAL, color);
-		gtk_widget_modify_bg (GTK_WIDGET (sia->applet),
-				      GTK_STATE_NORMAL, color);
-		break;
-
-	case PANEL_NO_BACKGROUND:
-		if (sia->ebox != NULL)
-			gtk_widget_modify_style (GTK_WIDGET (sia->ebox),
-						 rc_style);
-		gtk_widget_modify_style (GTK_WIDGET (sia->applet),
-					 rc_style);
-		break;
-
-	default:
-		if (sia->ebox != NULL)
-			gtk_widget_modify_style (GTK_WIDGET (sia->ebox),
-						 rc_style);
-		gtk_widget_modify_style (GTK_WIDGET (sia->applet),
-					 rc_style);
-		break;
-	}
-
+	GSwitchItAppletSetBackground(type, rc_style, sia->applet, color, pixmap);
 	gtk_rc_style_unref (rc_style);
 }
 
@@ -466,7 +476,7 @@ GSwitchItAppletKeyPressed (GtkWidget *
 	case GDK_Return:
 	case GDK_space:
 	case GDK_KP_Space:
-		GSwitchItAppletConfigLockNextGroup ();
+		GSwitchItConfigLockNextGroup ();
 		return TRUE;
 	default:
 		break;
@@ -482,30 +492,12 @@ GSwitchItAppletButtonPressed (GtkWidget *
 {
 	if (event->button == 1 && event->type == GDK_BUTTON_PRESS) {
 		XklDebug (150, "Mouse button pressed on applet\n");
-		GSwitchItAppletConfigLockNextGroup ();
+		GSwitchItConfigLockNextGroup ();
 		return TRUE;
 	}
 	return FALSE;
 }
 
-void
-GSwitchItAppletCmdProps (BonoboUIComponent *
-			 uic, GSwitchItApplet * sia, const gchar * verb)
-{
-	/* Only one preferences window at a time */
-	if (sia->propsDialog) {
-		gtk_window_set_screen (GTK_WINDOW (sia->propsDialog),
-				       gtk_widget_get_screen (GTK_WIDGET
-							      (sia->
-							       applet)));
-
-		gtk_window_present (GTK_WINDOW (sia->propsDialog));
-		return;
-	}
-
-	GSwitchItAppletPropsCreate (sia);
-}
-
 void
 GSwitchItAppletCmdCapplet (BonoboUIComponent *
 			   uic, GSwitchItApplet * sia, const gchar * verb)
@@ -572,7 +564,7 @@ void
 GSwitchItAppletCmdHelp (BonoboUIComponent
 			* uic, GSwitchItApplet * sia, const gchar * verb)
 {
-	GSwitchItHelp (GTK_WIDGET (sia->applet), "gswitchitApplet");
+	GSwitchItHelp (GTK_WIDGET (sia->applet), NULL);
 }
 
 void
@@ -587,48 +579,20 @@ GSwitchItAppletCmdAbout (BonoboUIComponent *
 		"Sergey V. Udaltsov<svu@gnome.org>",
 		NULL
 	};
-	gchar *translatorCredits;
-	GdkPixbuf *pixbuf = NULL;
-	gchar *file;
-	if (sia->aboutDialog) {
-		gtk_window_set_screen (GTK_WINDOW (sia->aboutDialog),
-				       gtk_widget_get_screen (GTK_WIDGET
-							      (sia->
-							       applet)));
-
-		gtk_window_present (GTK_WINDOW (sia->aboutDialog));
-		return;
-	}
-
-	file =
-	    gnome_program_locate_file (NULL,
-				       GNOME_FILE_DOMAIN_PIXMAP,
-				       "gswitchit-applet.png", TRUE, NULL);
-	pixbuf = gdk_pixbuf_new_from_file (file, NULL);
-	translatorCredits = _("translator_credits");
-	translatorCredits =
-	    strcmp (translatorCredits,
-		    "translator_credits") != 0 ? translatorCredits : NULL;
-	sia->aboutDialog =
-	    gnome_about_new (_("Keyboard Indicator"), VERSION,
+	
+	const gchar *translator_credits = _("translator_credits");
+		     
+	gtk_show_about_dialog (NULL,
+			       "name",         _("Keyboard Indicator"),
+			       "version",      VERSION,
 /* Translators: Please replace (C) with the proper copyright character. */
-			     _
-			     ("Copyright (c) Sergey V. Udaltsov 1999-2004"),
-			     _
-			     ("Keyboard layout indicator applet for GNOME"),
-			     authors, documenters, translatorCredits,
-			     pixbuf);
-	g_object_add_weak_pointer (G_OBJECT (sia->aboutDialog),
-				   (gpointer) & (sia->aboutDialog));
-	gnome_window_icon_set_from_file (GTK_WINDOW (sia->aboutDialog),
-					 file);
-	g_free (file);
-
-	gtk_window_set_screen (GTK_WINDOW (sia->aboutDialog),
-			       gtk_widget_get_screen (GTK_WIDGET
-						      (sia->applet)));
-
-	gtk_window_present (GTK_WINDOW (sia->aboutDialog));
+			       "copyright",    _("Copyright (c) Sergey V. Udaltsov 1999-2004"),
+			       "comments",     _("Keyboard layout indicator applet for GNOME"),
+			       "authors",      authors,
+			       "documenters",  documenters,
+			       "translator-credits",  strcmp (translator_credits, "translator_credits") != 0 ? translator_credits : NULL,
+			       "logo-icon-name",       "gswitchit-applet",
+			       NULL);
 }
 
 static void
@@ -638,7 +602,7 @@ GSwitchItAppletCleanupGroupsSubmenu (GSwitchItApplet * sia)
 	BonoboUIComponent *popup;
 	popup =
 	    panel_applet_get_popup_component (PANEL_APPLET (sia->applet));
-	for (i = XkbNumKbdGroups; --i >= 0;) {
+	for (i = 0;;i++) {
 		char path[80];
 		g_snprintf (path, sizeof (path),
 			    GROUPS_SUBMENU_PATH "/Group_%d", i);
@@ -647,7 +611,8 @@ GSwitchItAppletCleanupGroupsSubmenu (GSwitchItApplet * sia)
 			XklDebug (160,
 				  "Unregistered group menu item \'%s\'\n",
 				  path);
-		}
+		} else
+			break;
 	}
 
 	XklDebug (160, "Unregistered group submenu\n");
@@ -657,15 +622,12 @@ static void
 GSwitchItAppletSetupGroupsSubmenu (GSwitchItApplet * sia)
 {
 	unsigned i, nGroups;
-	char *pname;
+	GSList *nameNode = globals.groupNames;
 	BonoboUIComponent *popup;
-	GSList *layout;
 	popup =
 	    panel_applet_get_popup_component (PANEL_APPLET (sia->applet));
-	XklDebug (160, "Registered group submenu\n");
-	nGroups = XklGetNumGroups ();
-	pname = (char *) sia->groupNames;
-	layout = sia->xkbConfig.layouts;
+	XklDebug (160, "Registering group submenu\n");
+	nGroups = g_slist_length (globals.groupNames);
 	for (i = 0; i < nGroups; i++) {
 		char verb[40];
 		BonoboUINode *node;
@@ -673,18 +635,18 @@ GSwitchItAppletSetupGroupsSubmenu (GSwitchItApplet * sia)
 		node = bonobo_ui_node_new ("menuitem");
 		bonobo_ui_node_set_attr (node, "name", verb);
 		bonobo_ui_node_set_attr (node, "verb", verb);
-		bonobo_ui_node_set_attr (node, "label", pname);
+		bonobo_ui_node_set_attr (node, "label", nameNode->data);
 		bonobo_ui_node_set_attr (node, "pixtype", "filename");
-		if (sia->appletConfig.showFlags) {
-			const char *imageFile =
-			    GSwitchItAppletConfigGetImagesFile (&sia->
-								appletConfig,
-								&sia->
-								xkbConfig,
+		if (globals.appletConfig.showFlags) {
+			char *imageFile =
+			    GSwitchItAppletConfigGetImagesFile (&globals.appletConfig,
+								&globals.kbdConfig,
 								i);
-			if (imageFile != NULL)
+			if (imageFile != NULL) {
 				bonobo_ui_node_set_attr (node, "pixname",
 							 imageFile);
+				g_free (imageFile);
+			}
 		}
 		bonobo_ui_component_set_tree (popup, GROUPS_SUBMENU_PATH,
 					      node, NULL);
@@ -693,9 +655,8 @@ GSwitchItAppletSetupGroupsSubmenu (GSwitchItApplet * sia)
 					      sia);
 		XklDebug (160,
 			  "Registered group menu item \'%s\' as \'%s\'\n",
-			  verb, pname);
-		pname += sizeof (sia->groupNames[0]);
-		layout = g_slist_next (layout);
+			  verb, nameNode->data);
+		nameNode = g_slist_next (nameNode);
 	}
 }
 
@@ -703,34 +664,37 @@ static void
 GSwitchItAppletSetupMenu (GSwitchItApplet * sia)
 {
 	panel_applet_setup_menu_from_file
-	    (PANEL_APPLET (sia->applet), NULL,
+	    (PANEL_APPLET (sia->applet), DATADIR,
 	     "GNOME_GSwitchItApplet.xml", NULL, gswitchitAppletMenuVerbs,
 	     sia);
 	GSwitchItAppletSetupGroupsSubmenu (sia);
 }
 
+/* Should be called once for all applets */
 static void
 GSwitchItAppletStartListen (GSwitchItApplet * sia)
 {
 	gdk_window_add_filter (NULL,
 			       (GdkFilterFunc) GSwitchItAppletFilterXEvt,
-			       sia);
+			       NULL);
 	gdk_window_add_filter (gdk_get_default_root_window (),
 			       (GdkFilterFunc) GSwitchItAppletFilterXEvt,
-			       sia);
-	XklStartListen ();
+			       NULL);
+
+	XklStartListen (XKLL_TRACK_KEYBOARD_STATE);
 }
 
+/* Should be called once for all applets */
 static void
 GSwitchItAppletStopListen (GSwitchItApplet * sia)
 {
 	XklStopListen ();
-/* !! no client message filter removal in gnome 2.2 */
+
 	gdk_window_remove_filter (NULL, (GdkFilterFunc)
-				  GSwitchItAppletFilterXEvt, sia);
+				  GSwitchItAppletFilterXEvt, NULL);
 	gdk_window_remove_filter
 	    (gdk_get_default_root_window (),
-	     (GdkFilterFunc) GSwitchItAppletFilterXEvt, sia);
+	     (GdkFilterFunc) GSwitchItAppletFilterXEvt, NULL);
 }
 
 static void GSwitchItAppletTerm (PanelApplet *
@@ -743,13 +707,14 @@ GSwitchItAppletInit (GSwitchItApplet * sia, PanelApplet * applet)
 	GtkNotebook *notebook;
 	GConfClient *confClient;
 
+	XklDebug (100, "Starting the applet startup process for %p\n", sia);
 	glade_gnome_init ();
 
 	sia->applet = GTK_WIDGET (applet);
 
 	panel_applet_set_flags (applet, PANEL_APPLET_EXPAND_MINOR);
 
-	notebook = GTK_NOTEBOOK (gtk_notebook_new ());
+	notebook = GTK_NOTEBOOK (sia->notebook = gtk_notebook_new ());
 	gtk_notebook_set_show_tabs (notebook, FALSE);
 	gtk_notebook_set_show_border (notebook, FALSE);
 
@@ -766,63 +731,79 @@ GSwitchItAppletInit (GSwitchItApplet * sia, PanelApplet * applet)
 	GSwitchItAppletSetTooltip (sia, _(PACKAGE));
 	gtk_widget_show_all (sia->applet);
 	gtk_widget_realize (sia->applet);
-	/* GSwitchItInstallGlibLogAppender(  ); */
-	if (XklInit (GDK_DISPLAY ()) != 0) {
-		GSwitchItAppletSetTooltip (sia,
-					   _("XKB initialization error"));
-		return TRUE;
+
+	sia->appletAncestor = gtk_widget_get_ancestor (sia->applet,
+						       GTK_TYPE_WINDOW);
+
+	if (GSwitchItAppletFirstInstance ()) {
+		/* GSwitchItInstallGlibLogAppender(  ); */
+		if (XklInit (GDK_DISPLAY ()) != 0) {
+			GSwitchItAppletSetTooltip (sia,
+						   _("XKB initialization error"));
+			return TRUE;
+		}
+		XklDebug (100, "*** First instance *** \n");
+
+		XklConfigInit ();
+		if (!XklConfigLoadRegistry ()) {
+			GSwitchItAppletSetTooltip (sia,
+						   _
+						   ("Error loading XKB configuration registry"));
+			return TRUE;
+		}
+
+		confClient = gconf_client_get_default ();
+
+		memset (&globals, 0, sizeof(globals));
+
+		XklRegisterStateCallback ((XklStateCallback)
+					  GSwitchItAppletStateCallback, NULL);
+		XklRegisterConfigCallback ((XklConfigCallback)
+					   GSwitchItAppletKbdConfigCallback, NULL);
+		GSwitchItPluginContainerInit (&globals.pluginContainer, confClient);
+
+
+		GSwitchItConfigInit (&globals.config, confClient);
+		GSwitchItKbdConfigInit (&globals.kbdConfig, confClient);
+		GSwitchItAppletConfigInit (&globals.appletConfig, confClient);
+
+		g_object_unref (confClient);
+
+		GSwitchItConfigLoadFromGConf (&globals.config);
+		GSwitchItConfigActivate (&globals.config);
+		GSwitchItKbdConfigLoadFromXCurrent (&globals.kbdConfig);
+		GSwitchItAppletConfigLoadFromGConf (&globals.appletConfig);
+		GSwitchItAppletConfigUpdateImages (&globals.appletConfig, &globals.kbdConfig);
+		GSwitchItAppletConfigActivate (&globals.appletConfig);
+
+		globals.groupNames = GSwitchItConfigLoadGroupDescriptionsUtf8 (&globals.config);
+
+		GSwitchItPluginManagerInit (&globals.pluginManager);
+		GSwitchItPluginManagerInitEnabledPlugins (&globals.pluginManager,
+							  &globals.pluginContainer,
+							  globals.appletConfig.enabledPlugins);
+		GSwitchItConfigStartListen (&globals.config,
+					    (GConfClientNotifyFunc)
+					    GSwitchItConfigChanged,
+					    NULL);
+		GSwitchItAppletConfigStartListen (&globals.appletConfig,
+						  (GConfClientNotifyFunc)
+						  GSwitchItAppletConfigChanged,
+						  NULL);
+		GSwitchItAppletStartListen (sia);
+
+		XklDebug (100, "*** First instance inited globals *** \n");
 	}
-
-	XklConfigInit ();
-	if (!XklConfigLoadRegistry ()) {
-		GSwitchItAppletSetTooltip (sia,
-					   _
-					   ("Error loading XKB configuration registry"));
-		return TRUE;
-	}
-
-	XklRegisterWindowCallback ((XklWinCallback)
-				   GSwitchItAppletWindowCallback,
-				   (void *) sia);
-	XklRegisterStateCallback ((XklStateCallback)
-				  GSwitchItAppletStateCallback,
-				  (void *) sia);
-	XklRegisterConfigCallback ((XklConfigCallback)
-				   GSwitchItAppletXkbConfigCallback,
-				   (void *) sia);
-
-	confClient = gconf_client_get_default ();
-	GSwitchItPluginContainerInit (&sia->pluginContainer, confClient);
-	g_object_unref (confClient);
-
-	GSwitchItXkbConfigInit (&sia->xkbConfig, confClient);
-	GSwitchItAppletConfigInit (&sia->appletConfig, confClient);
-	GSwitchItPluginManagerInit (&sia->pluginManager);
-	GSwitchItXkbConfigLoadCurrent (&sia->xkbConfig);
-	GSwitchItAppletConfigLoad (&sia->appletConfig);
-	GSwitchItAppletConfigUpdateImages (&sia->appletConfig,
-					   &sia->xkbConfig);
-	GSwitchItAppletConfigActivate (&sia->appletConfig);
-	GSwitchItAppletConfigLoadGroupDescriptionsUtf8 (&sia->appletConfig,
-							sia->groupNames);
-	GSwitchItPluginManagerInitEnabledPlugins (&sia->pluginManager,
-						  &sia->pluginContainer,
-						  sia->appletConfig.
-						  enabledPlugins);
-	GSwitchItAppletConfigStartListen (&sia->appletConfig,
-					  (GConfClientNotifyFunc)
-					  GSwitchItAppletConfigChanged,
-					  sia);
 	GSwitchItAppletFillNotebook (sia);
 	GSwitchItAppletRevalidate (sia);
-	g_signal_connect (G_OBJECT (sia->applet), "change_size",
-			  G_CALLBACK (GSwitchItAppletChangePixelSize),
-			  sia);
-	g_signal_connect (G_OBJECT (sia->applet), "change_background",
-			  G_CALLBACK (GSwitchItAppletChangeBackground),
-			  sia);
 
-	GSwitchItAppletStartListen (sia);
+	g_signal_connect (G_OBJECT (sia->applet), "change_size",
+			  G_CALLBACK (GSwitchItAppletChangePixelSize), sia);
+	g_signal_connect (G_OBJECT (sia->applet), "change_background",
+			  G_CALLBACK (GSwitchItAppletChangeBackground), sia);
+	g_signal_connect (G_OBJECT (sia->applet), "button_press_event",
+			  G_CALLBACK (GSwitchItAppletButtonPressed), sia);
+
 	gtk_widget_add_events (sia->applet, GDK_BUTTON_PRESS_MASK);
 
 	g_signal_connect (GTK_OBJECT (sia->applet), "destroy",
@@ -835,32 +816,38 @@ GSwitchItAppletInit (GSwitchItApplet * sia, PanelApplet * applet)
 void
 GSwitchItAppletTerm (PanelApplet * applet, GSwitchItApplet * sia)
 {
-	if (terminatedOnce) {
-		XklDebug (0,
-			  "Please do not call the termination method twice!!!\n");
-		return;
+	XklDebug (100, "Starting the applet shutdown process for %p\n", sia);
+	/* remove BEFORE all termination work is finished */
+	globals.appletInstances = g_slist_remove (globals.appletInstances, sia);
+
+	if (GSwitchItAppletLastInstance ()) {
+		XklDebug (100, "*** Last instance *** \n");
+		GSwitchItAppletStopListen (sia);
+
+		GSwitchItConfigStopListen (&globals.config);
+		GSwitchItAppletConfigStopListen (&globals.appletConfig);
+
+		XklRegisterStateCallback (NULL, NULL);
+		XklRegisterConfigCallback (NULL, NULL);
+
+		GSwitchItPluginManagerTermInitializedPlugins (&globals.pluginManager);
+		GSwitchItPluginManagerTerm (&globals.pluginManager);
+
+		GSwitchItAppletConfigTerm (&globals.appletConfig);
+		GSwitchItKbdConfigTerm (&globals.kbdConfig);
+		GSwitchItConfigTerm (&globals.config);
+
+		GSwitchItPluginContainerTerm (&globals.pluginContainer);
+
+		XklConfigFreeRegistry ();
+		XklConfigTerm ();
+		XklDebug (100, "*** Last instance terminated globals *** \n");
+		XklTerm ();
 	}
-	terminatedOnce = TRUE;
-	XklDebug (100, "Starting the applet shutdown process\n");
-	GSwitchItAppletStopListen (sia);
-	GSwitchItAppletConfigStopListen (&sia->appletConfig);
-	XklRegisterStateCallback (NULL, NULL);
-	XklRegisterConfigCallback (NULL, NULL);
-	XklRegisterWindowCallback (NULL, NULL);
-	GSwitchItPluginManagerTermInitializedPlugins (&sia->pluginManager);
-	GSwitchItPluginManagerTerm (&sia->pluginManager);
-	GSwitchItAppletConfigTerm (&sia->appletConfig);
-	GSwitchItXkbConfigTerm (&sia->xkbConfig);
-	GSwitchItPluginContainerTerm (&sia->pluginContainer);
+
 	GSwitchItAppletCleanupNotebook (sia);
 
-	XklConfigFreeRegistry ();
-	XklConfigTerm ();
-	XklTerm ();
-
-	terminatedOnce = TRUE;
-	g_free (theAppletInstance);
-	theAppletInstance = NULL;
+	g_free (sia);
 	XklDebug (100, "The applet successfully terminated\n");
 }
 
@@ -868,20 +855,17 @@ gboolean
 GSwitchItAppletNew (PanelApplet * applet)
 {
 	gboolean rv = TRUE;
+	GSwitchItApplet * sia;
 #if 0
 	GLogLevelFlags fatal_mask;
 	fatal_mask = G_LOG_LEVEL_WARNING | G_LOG_LEVEL_CRITICAL;
 	g_log_set_always_fatal (fatal_mask);
 #endif
-	if (theAppletInstance == NULL) {
-		terminatedOnce = FALSE;
-		theAppletInstance = g_new0 (GSwitchItApplet, 1);
-		rv = GSwitchItAppletInit (theAppletInstance, applet);
-		XklDebug (100, "The applet successfully started: %d\n",
-			  rv);
-	} else
-		XklDebug (0,
-			  "Please do not call the initialization method twice!!!\n");
+	sia = g_new0 (GSwitchItApplet, 1);
+	rv = GSwitchItAppletInit (sia, applet);
+	/* append AFTER all initialization work is finished */
+	globals.appletInstances = g_slist_append (globals.appletInstances, sia);
+	XklDebug (100, "The applet successfully started: %d\n", rv);
 	return rv;
 }
 
@@ -889,16 +873,13 @@ GSwitchItAppletNew (PanelApplet * applet)
 void
 GSwitchItPluginContainerReinitUi (GSwitchItPluginContainer * pc)
 {
-	GSwitchItAppletReinitUi ((GSwitchItApplet *) pc);
+	ForAllApplets ()
+		GSwitchItAppletReinitUi (sia);
+	NextApplet
 }
 
-void
-GSwitchItPluginLoadLocalizedGroupNames (GSwitchItPluginContainer
-					* pc,
-					const GroupDescriptionsBuffer
-					** outDescr)
+GSList *
+GSwitchItPluginLoadLocalizedGroupNames (GSwitchItPluginContainer * pc)
 {
-	*outDescr =
-	    (const GroupDescriptionsBuffer
-	     *) (&(((GSwitchItApplet *) pc)->groupNames));
+	return globals.groupNames;
 }
