@@ -1,4 +1,4 @@
-/* $Id: gweather-applet.c,v 1.87.2.1 2004/10/13 12:06:46 kmaraas Exp $ */
+/* $Id: gweather-applet.c,v 1.93 2005/02/09 12:57:14 kmaraas Exp $ */
 
 /*
  *  Papadimitriou Spiros <spapadim+@cs.cmu.edu>
@@ -29,6 +29,8 @@
 #include "gweather-dialog.h"
 #include "gweather-applet.h"
 
+#define MAX_CONSECUTIVE_FAULTS (3)
+
 static void place_widgets (GWeatherApplet *gw_applet)
 {
     GtkRequisition req;
@@ -133,28 +135,39 @@ static void change_background_cb (
 				GdkColor *color, GdkPixmap *pixmap, 
 				gpointer data)
 {
+	/* taken from the TrashApplet */
 	GWeatherApplet *gw_applet = (GWeatherApplet *) data;
-	GtkRcStyle *rc_style = gtk_rc_style_new ();
+	GtkRcStyle *rc_style;
+	GtkStyle *style;
+
+	/* reset style */
+	gtk_widget_set_style (GTK_WIDGET (gw_applet->applet), NULL);
+	rc_style = gtk_rc_style_new ();
+	gtk_widget_modify_style (GTK_WIDGET (gw_applet->applet), rc_style);
+	g_object_unref (rc_style);
 
 	switch (type) {
-		case PANEL_PIXMAP_BACKGROUND:
-			gtk_widget_modify_style (GTK_WIDGET (gw_applet->applet), rc_style);
+		case PANEL_COLOR_BACKGROUND:
+			gtk_widget_modify_bg (GTK_WIDGET (gw_applet->applet),
+					GTK_STATE_NORMAL, color);
 			break;
 
-		case PANEL_COLOR_BACKGROUND:
-			gtk_widget_modify_bg (GTK_WIDGET (gw_applet->applet), GTK_STATE_NORMAL, color);
+		case PANEL_PIXMAP_BACKGROUND:
+			style = gtk_style_copy (
+					GTK_WIDGET (gw_applet->applet)->style);
+			if (style->bg_pixmap[GTK_STATE_NORMAL])
+				g_object_unref
+					(style->bg_pixmap[GTK_STATE_NORMAL]);
+			style->bg_pixmap[GTK_STATE_NORMAL] = g_object_ref
+				(pixmap);
+			gtk_widget_set_style (GTK_WIDGET (gw_applet->applet),
+					style);
 			break;
 
 		case PANEL_NO_BACKGROUND:
-			gtk_widget_modify_style (GTK_WIDGET (gw_applet->applet), rc_style);
-			break;
-
 		default:
-			gtk_widget_modify_style (GTK_WIDGET (gw_applet->applet), rc_style);
 			break;
 	}
-
-	gtk_rc_style_unref (rc_style);
 }
 
 
@@ -166,7 +179,11 @@ static gboolean clicked_cb (GtkWidget *widget, GdkEventButton *ev, gpointer data
         return FALSE;
 
     if (ev->type == GDK_BUTTON_PRESS) {
-	gweather_dialog_open(gw_applet);
+	if (!gw_applet->gweather_dialog)
+		gweather_dialog_open (gw_applet);
+	else
+		gweather_dialog_close (gw_applet);
+	
 	return TRUE;
     }
     
@@ -309,8 +326,6 @@ applet_destroy (GtkWidget *widget, GWeatherApplet *gw_applet)
 
 void gweather_applet_create (GWeatherApplet *gw_applet)
 {
-    GtkWidget *label;
-    GtkWidget *pixmap;
     GtkTooltips *tooltips;
     GtkIconInfo *icon_info;
     AtkObject *atk_obj;
@@ -364,7 +379,7 @@ void gweather_applet_create (GWeatherApplet *gw_applet)
     gw_applet->orient = panel_applet_get_orient (gw_applet->applet);
     
     panel_applet_setup_menu_from_file (gw_applet->applet,
-                                       NULL,
+                                       DATADIR,
 			               "GNOME_GWeatherApplet.xml",
                                        NULL,
 			               weather_applet_menu_verbs,
@@ -435,32 +450,48 @@ gint timeout_cb (gpointer data)
 
 void update_finish (WeatherInfo *info)
 {
+    static int gw_fault_counter = 0;
+    
     char *s;
     GWeatherApplet *gw_applet = info->applet;
-   
-    weather_info_get_pixbuf_mini(gw_applet->gweather_info, 
-    				 &(gw_applet->applet_pixbuf));
-    gtk_image_set_from_pixbuf (GTK_IMAGE (gw_applet->image), 
-    			       gw_applet->applet_pixbuf);
-      
-    gtk_label_set_text(GTK_LABEL(gw_applet->label), 
-			weather_info_get_temp_summary(gw_applet->gweather_info));
-    
-    s = weather_info_get_weather_summary(gw_applet->gweather_info);
-    gtk_tooltips_set_tip(gw_applet->tooltips, GTK_WIDGET(gw_applet->applet), s, NULL);
-    g_free (s);
-
-
-    /* Update timer */
-    if (gw_applet->timeout_tag > 0)
-        gtk_timeout_remove(gw_applet->timeout_tag);
-    if (gw_applet->gweather_pref.update_enabled)
-        gw_applet->timeout_tag =  
-        	gtk_timeout_add (gw_applet->gweather_pref.update_interval * 1000,
-                                 timeout_cb, gw_applet);
-
-    /* Update dialog -- if one is present */
-    gweather_dialog_update(gw_applet);
+  
+    if (FALSE == info->valid)
+    {
+	    /* there has been an error during retrival
+	     * just update the fault counter
+	     */
+	    gw_fault_counter++;
+    }
+    else if ((TRUE == info->valid) ||
+	     (gw_fault_counter >= MAX_CONSECUTIVE_FAULTS))
+    {
+	    gw_fault_counter = 0;
+	    weather_info_get_pixbuf_mini (gw_applet->gweather_info, 
+	    				 &(gw_applet->applet_pixbuf));
+	    gtk_image_set_from_pixbuf (GTK_IMAGE (gw_applet->image), 
+	    			       gw_applet->applet_pixbuf);
+	      
+	    gtk_label_set_text (GTK_LABEL (gw_applet->label), 
+	        		weather_info_get_temp_summary(
+					gw_applet->gweather_info));
+	    
+	    s = weather_info_get_weather_summary (gw_applet->gweather_info);
+	    gtk_tooltips_set_tip (gw_applet->tooltips, GTK_WIDGET (
+				    gw_applet->applet), s, NULL);
+	    g_free (s);
+	    
+	    /* Update timer */
+	    if (gw_applet->timeout_tag > 0)
+	        gtk_timeout_remove(gw_applet->timeout_tag);
+	    if (gw_applet->gweather_pref.update_enabled)
+	        gw_applet->timeout_tag =  
+	        	gtk_timeout_add (
+				gw_applet->gweather_pref.update_interval * 1000,
+	                        timeout_cb, gw_applet);
+	    
+	    /* Update dialog -- if one is present */
+	    gweather_dialog_update(gw_applet);
+    }
 }
 
 void gweather_update (GWeatherApplet *gw_applet)
