@@ -1,7 +1,7 @@
 #ifndef __WEATHER_H_
 #define __WEATHER_H_
 
-/* $Id: weather.h,v 1.29 2005/01/13 12:03:09 davyd Exp $ */
+/* $Id: weather.h,v 1.31.2.1 2005/03/30 06:17:42 ryanl Exp $ */
 
 /*
  *  Papadimitriou Spiros <spapadim+@cs.cmu.edu>
@@ -28,8 +28,7 @@ G_BEGIN_DECLS
 #define WEATHER_LOCATION_CODE_LEN 4
 
 struct _WeatherLocation {
-    gchar *untrans_name;
-    gchar *trans_name;
+    gchar *name;
     gchar *code;
     gchar *zone;
     gchar *radar;
@@ -42,9 +41,12 @@ struct _WeatherLocation {
 
 
 
-extern WeatherLocation *weather_location_new (const gchar *untrans_name, const gchar *trans_name, const gchar *code,
-					      const gchar *zone, const gchar *radar, const gchar *coordinates);
-extern WeatherLocation *weather_location_clone (const WeatherLocation *location);
+WeatherLocation *
+weather_location_new (const gchar *trans_name, const gchar *code,
+                      const gchar *zone, const gchar *radar,
+                      const gchar *coordinates);
+
+WeatherLocation *weather_location_clone (const WeatherLocation *location);
 extern void weather_location_free (WeatherLocation *location);
 extern gboolean weather_location_equal (const WeatherLocation *location1, const WeatherLocation *location2);
 
@@ -263,7 +265,7 @@ extern void _weather_info_get_pixbuf (WeatherInfo *info, gboolean mini, GdkPixbu
 #define PRESSURE_INCH_TO_HPA(inch)   ((inch) * 33.86)
 #define PRESSURE_INCH_TO_MM(inch)    ((inch) * 25.40005)
 #define PRESSURE_INCH_TO_MB(inch)    (PRESSURE_INCH_TO_HPA(inch))
-#define PRESSURE_MBAR_TO_INCH(mbar) ((mbar) * 0.02963742)
+#define PRESSURE_MBAR_TO_INCH(mbar) ((mbar) * 0.029533373)
 
 #define VISIBILITY_SM_TO_KM(sm)  ((sm) * 1.609344)
 #define VISIBILITY_SM_TO_M(sm)   (VISIBILITY_SM_TO_KM(sm) * 1000)
@@ -279,7 +281,7 @@ extern void bom_start_open (WeatherInfo *info);
 extern void wx_start_open (WeatherInfo *info);
 
 extern gboolean requests_init (WeatherInfoFunc cb, WeatherInfo *info);
-extern void requests_done (GnomeVFSAsyncHandle *handle, WeatherInfo *info);
+extern void request_done (GnomeVFSAsyncHandle *handle, WeatherInfo *info);
 extern void requests_done_check (WeatherInfo *info);
 
 G_END_DECLS
