@@ -46,8 +46,6 @@ const char GSWITCHIT_CONFIG_KEY_HANDLE_INDICATORS[] =
 const char GSWITCHIT_CONFIG_KEY_LAYOUT_NAMES_AS_GROUP_NAMES[]
     = GSWITCHIT_CONFIG_KEY_PREFIX "/layoutNamesAsGroupNames";
 
-static int gconfConfigListenerId;
-
 /**
  * GSwitchItAppletConfig
  */
@@ -61,8 +59,6 @@ const char GSWITCHIT_APPLET_CONFIG_KEY_ENABLED_PLUGINS[] =
 const char GSWITCHIT_APPLET_CONFIG_KEY_SECONDARIES[] =
     GSWITCHIT_APPLET_CONFIG_KEY_PREFIX "/secondary";
 
-static int gconfAppletConfigListenerId = 0;
-
 /**
  * GSwitchItKbdConfig
  */
@@ -101,8 +97,6 @@ const char *GSWITCHIT_KBD_CONFIG_SYSBACKUP[] = {
 	GSWITCHIT_KBD_CONFIG_SYSBACKUP_KEY_OPTIONS
 };
 
-static int gconfKbdConfigListenerId;
-
 /**
  * static common functions
  */
@@ -424,6 +418,7 @@ _GSwitchItKbdConfigLoadParams (GSwitchItKbdConfig * kbdConfig,
 			g_warning ("Error reading configuration:%s\n",
 				   gerror->message);
 			g_error_free (gerror);
+			g_free (pc);
 			gerror = NULL;
 		}
 		GSwitchItKbdConfigModelSet (kbdConfig, NULL);
@@ -642,21 +637,21 @@ GSwitchItConfigActivate (GSwitchItConfig * config)
 }
 
 void
-GSwitchItConfigLockNextGroup ()
+GSwitchItConfigLockNextGroup (void)
 {
 	int group = XklGetNextGroup ();
 	XklLockGroup (group);
 }
 
 void
-GSwitchIttConfigLockPrevGroup ()
+GSwitchIttConfigLockPrevGroup (void)
 {
 	int group = XklGetPrevGroup ();
 	XklLockGroup (group);
 }
 
 void
-GSwitchItConfigRestoreGroup ()
+GSwitchItConfigRestoreGroup (void)
 {
 	int group = XklGetRestoreGroup ();
 	XklLockGroup (group);
@@ -669,14 +664,14 @@ GSwitchItConfigStartListen (GSwitchItConfig * config,
 {
 	_GSwitchItConfigAddListener (config->confClient,
 				     GSWITCHIT_CONFIG_DIR, func,
-				     userData, &gconfConfigListenerId);
+				     userData, &config->configListenerId);
 }
 
 void
 GSwitchItConfigStopListen (GSwitchItConfig * config)
 {
 	_GSwitchItConfigRemoveListener (config->confClient,
-					&gconfConfigListenerId);
+					&config->configListenerId);
 }
 /**
  * extern GSwitchItKbdConfig config functions
@@ -922,14 +917,14 @@ GSwitchItKbdConfigStartListen (GSwitchItKbdConfig * kbdConfig,
 {
 	_GSwitchItConfigAddListener (kbdConfig->confClient,
 				     GSWITCHIT_KBD_CONFIG_DIR, func,
-				     userData, &gconfKbdConfigListenerId);
+				     userData, &kbdConfig->configListenerId);
 }
 
 void
 GSwitchItKbdConfigStopListen (GSwitchItKbdConfig * kbdConfig)
 {
 	_GSwitchItConfigRemoveListener (kbdConfig->confClient,
-					&gconfKbdConfigListenerId);
+					&kbdConfig->configListenerId);
 }
 
 /**
@@ -1014,7 +1009,19 @@ GSwitchItAppletConfigLoadImages (GSwitchItAppletConfig * appletConfig,
 			image =
 			    gdk_pixbuf_new_from_file (imageFile, &err);
 			if (image == NULL) {
-				gnome_error_dialog (err->message);
+				GtkWidget *dialog = gtk_message_dialog_new (NULL, 
+									    GTK_DIALOG_DESTROY_WITH_PARENT,
+									    GTK_MESSAGE_ERROR,
+									    GTK_BUTTONS_OK,
+									    _("There was an error loading an image: %s"),
+									    err->message);
+				g_signal_connect (G_OBJECT (dialog),
+						  "response",
+						  G_CALLBACK (gtk_widget_destroy), NULL);
+
+				gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
+
+				gtk_widget_show (dialog);
 				g_error_free (err);
 			}
 			XklDebug (150,
@@ -1187,14 +1194,14 @@ GSwitchItAppletConfigStartListen (GSwitchItAppletConfig * appletConfig,
 {
 	_GSwitchItConfigAddListener (appletConfig->confClient,
 				     GSWITCHIT_APPLET_CONFIG_DIR, func,
-				     userData, &gconfAppletConfigListenerId);
+				     userData, &appletConfig->configListenerId);
 }
 
 void
 GSwitchItAppletConfigStopListen (GSwitchItAppletConfig * appletConfig)
 {
 	_GSwitchItConfigRemoveListener (appletConfig->confClient,
-					&gconfAppletConfigListenerId);
+					&appletConfig->configListenerId);
 }
 
 Bool
@@ -1255,10 +1262,24 @@ GSwitchItConfigLoadGroupDescriptionsUtf8 (GSwitchItConfig * config)
 					     (lDescr, vDescr), -1, NULL,
 					     NULL, NULL);
 					rv = g_slist_append (rv, nameUtf);
+				} else {
+					rv = g_slist_append (rv, g_strdup(""));
 				}
 			}
 		}
 		XklConfigRecDestroy (&xklConfig);
+		/* Worst case - multiple layous - but SOME of them are multigrouped :((( 
+		   We cannot do much - just add empty descriptions. 
+		   The UI is going to be messy. 
+		   Canadian layouts are famous for this sh.t. */
+		if (g_slist_length (rv) != XklGetNumGroups())
+		{
+			XklDebug (0, "The mismatch between "
+				  "the number of groups: %d and number of layouts: %d\n",
+				  XklGetNumGroups(), g_slist_length (rv));
+			for (i = XklGetNumGroups() - g_slist_length (rv); --i>=0;)
+				rv = g_slist_append (rv, g_strdup(""));
+		}
 	}
 	if (rv == NULL)
 		for (i = XklGetNumGroups (); --i >= 0;)
