@@ -24,11 +24,7 @@
 
 G_BEGIN_DECLS
 
-#define GNOME_VOLUME_APPLET_KEY_DIR \
-  "/apps/gnome-volume-applet"
-#define GNOME_VOLUME_APPLET_KEY(key) \
-  GNOME_VOLUME_APPLET_KEY_DIR "/" key
-
+#define GNOME_VOLUME_APPLET_KEY(x) x
 #define GNOME_VOLUME_APPLET_KEY_ACTIVE_ELEMENT \
   GNOME_VOLUME_APPLET_KEY ("active-element")
 #define GNOME_VOLUME_APPLET_KEY_ACTIVE_TRACK \
