@@ -18,17 +18,13 @@
 #include <dirent.h>
 #include <string.h>
 #include <time.h>
-#include <gnome.h>
-#include <panel-applet.h>
-#include <gconf/gconf-client.h>
-#include <panel-applet-gconf.h>
 
+#include <gdk/gdkkeysyms.h>
 #include <gtk/gtk.h>
-#include <libgnomeui/libgnomeui.h>
-#include <libgnome/libgnome.h>
-#include <libgnomeui/gnome-window-icon.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
-#include <libart_lgpl/libart.h>
+#include <gconf/gconf-client.h>
+#include <panel-applet.h>
+#include <panel-applet-gconf.h>
 
 #include "global.h"
 
@@ -41,7 +37,7 @@ about_cb (BonoboUIComponent *uic,
     {
 		"Martin Baulig <martin@home-of-linux.org>",
 		"Todd Kulesza <fflewddur@dropline.net>",
-		"Benoît Dejean <benoit.dejean@placenet.org>",
+		"Benoît Dejean <TazForEver@dlfp.org>",
 		"Davyd Madeley <davyd@madeley.id.au>",
 		NULL
     };
@@ -171,7 +167,7 @@ multiload_change_background_cb (PanelApplet *a,
 	gtk_widget_set_style (GTK_WIDGET (ma->applet), NULL);
 	rc_style = gtk_rc_style_new ();
 	gtk_widget_modify_style (GTK_WIDGET (ma->applet), rc_style);
-	g_object_unref (rc_style);
+	gtk_rc_style_unref (rc_style);
 
 	switch (type) {
 		case PANEL_COLOR_BACKGROUND:
@@ -187,6 +183,7 @@ multiload_change_background_cb (PanelApplet *a,
 			style->bg_pixmap[GTK_STATE_NORMAL] = g_object_ref
 				(pixmap);
 			gtk_widget_set_style (GTK_WIDGET (ma->applet), style);
+			g_object_unref (style);
 			break;
 
 		case PANEL_NO_BACKGROUND:
