@@ -20,13 +20,18 @@
 #ifndef __STICKYNOTES_APPLET_H__
 #define __STICKYNOTES_APPLET_H__
 
-#include <gnome.h>
+#include <gdk/gdkscreen.h>
+#include <gdk-pixbuf/gdk-pixbuf.h>
+#include <gtk/gtkadjustment.h>
+#include <gtk/gtktooltips.h>
+#include <gtk/gtkwidget.h>
+#include <glade/glade-xml.h>
 #include <panel-applet.h>
 #include <gconf/gconf-client.h>
 #include <glade/glade.h>
 
 #define GCONF_PATH	"/apps/stickynotes_applet"
-#define GLADE_PATH	STICKYNOTES_GLADEDIR "/stickynotes.glade"
+#define GLADE_PATH	GNOME_GLADEDIR "/stickynotes.glade"
 #define XML_PATH	"/.gnome2/stickynotes_applet"
 #define ICON_PATH	STICKYNOTES_ICONDIR
 
@@ -45,10 +50,10 @@ typedef struct
 	GtkAdjustment *w_prefs_width;
 	GtkAdjustment *w_prefs_height;
 	GtkWidget *w_prefs_color;
+	GtkWidget *w_prefs_font_color;
 	GtkWidget *w_prefs_sys_color;
 	GtkWidget *w_prefs_font;
 	GtkWidget *w_prefs_sys_font;
-	GtkWidget *w_prefs_click;
 	GtkWidget *w_prefs_sticky;
 	GtkWidget *w_prefs_force;
 
@@ -60,7 +65,6 @@ typedef struct
 
 	GConfClient *gconf;		/* GConf Client */
 	GtkTooltips *tooltips;		/* Tooltips */
-
 } StickyNotes;
 
 /* Sticky Notes Icons */
@@ -78,10 +82,17 @@ typedef struct
 	GtkWidget *w_image;		/* The applet icon */
 
 	GtkWidget *about_dialog;	/* The applet it's about dialog */
-
+	GtkWidget *destroy_all_dialog;	/* The applet it's destroy all dialog */
+	
 	gboolean prelighted;		/* Whether applet is prelighted */
 	gboolean pressed;		/* Whether applet is pressed */
+
+	gint panel_size;
+	PanelAppletOrient panel_orient;
 	
+	GtkWidget *popup_menu;			/* Popup Menu */
+	GtkWidget *menu_tip;
+	GtkWidget *menu_show;
 } StickyNotesApplet;
 	
 typedef enum
@@ -95,15 +106,15 @@ typedef enum
 extern StickyNotes *stickynotes;
 
 void stickynotes_applet_init(PanelApplet *panel_applet);
-void stickynotes_applet_init_icons();
-void stickynotes_applet_init_prefs();
+void stickynotes_applet_init_icons(void);
+void stickynotes_applet_init_prefs(void);
 
 StickyNotesApplet * stickynotes_applet_new(PanelApplet *panel_applet);
 
 void stickynotes_applet_update_icon(StickyNotesApplet *applet);
-void stickynotes_applet_update_prefs();
-void stickynotes_applet_update_menus();
-void stickynotes_applet_update_tooltips();
+void stickynotes_applet_update_prefs(void);
+void stickynotes_applet_update_menus(void);
+void stickynotes_applet_update_tooltips(void);
 
 void stickynotes_applet_do_default_action(GdkScreen *screen);
 
